unit FrameItem;

interface

uses
{$IFDEF ANDROID}
  Androidapi.JNI.Media,
  Androidapi.Helpers,
  Androidapi.JNIBridge,
  AndroidApi.Jni.JavaTypes,
  AndroidApi.Jni.App,
{$ENDIF}
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Controls.Presentation, FMX.Objects;

type

  TfrItem = class;

  TFrameType = (ftNuevo, ftModif);

  TVolumeValue = record
    Value:single;
    MaxValue:single;
  end;

  PVolumeValues = ^TVolumeValues;

  TVolumeValues = record
    Notificaciones:TVolumeValue;
    Multimedia:TVolumeValue;
    Alarma:TVolumeValue;
    Voz:TVolumeValue;
  end;

  TStreamType = (stNotificaciones, stMultimedia, stAlarma, stVoz);

  TPerfil = class(TObject)
  private
    FNombre: string;
  public
    FVolumeValues: TVolumeValues;
    property Nombre:string read FNombre write FNombre;
    property VolumeValues:TVolumeValues read FVolumeValues write FVolumeValues;
  end;

  TPerfiles = class(TStringList)
  private
    FIniFile: String;

    function GetPerfilByIndex(Index: integer): TPerfil;
    function GetNombreByIndex(Index: integer): string;
  public
    constructor Create(pIniFile:String);

    property Nombre[Index:integer]:string read GetNombreByIndex;
    property Perfil[Index:integer]:TPerfil read GetPerfilByIndex;
    property IniFile:String read FIniFile write FIniFile;

    function GetPerfilByName(pName:string):TPerfil;

    procedure AddPerfil(pPerfil:TPerfil);
    procedure UpdatePerfil(pPerfil:TPerfil);
    procedure DeletePerfil(pPerfil:TPerfil);

    procedure LoadFromFile();
    procedure SaveToFile();

  end;

{
  TListaPerfiles = class(TStringList)
  private
    function GetNombre(Index:integer): string;
    procedure SetNombre(Index:integer; const Value: string);
    function GetFrame(Index: integer): TfrItem;
    procedure SetFrame(Index: integer; const Value: TfrItem);
  public
    property Nombre[Index:integer]:string read GetNombre write SetNombre;
    property Frame[Index:integer]:TfrItem read GetFrame write SetFrame;

    procedure AddFrame(pNombre:string; pFrame:TfrItem);
  end;

  PFrame = ^TfrItem;
}

  TfrItem = class(TFrame)
    lblTitulo: TLabel;
    lblPorcentage: TLabel;
    tbValor: TTrackBar;
    btnAplicar: TButton;
    Image1: TImage;
    Rectangle1: TRectangle;
    Rectangle2: TRectangle;
    Rectangle3: TRectangle;
    procedure tbValorChange(Sender: TObject);
  private
//    FStreamType: TStreamType;
//    FFrameType: TFrameType;
    function GetTitulo: string;
    function GetValor: single;
    procedure SetTitulo(const Value: string);
    procedure SetValor(const Value: single);
    function GetValorMax: single;
    procedure SetValorMax(const Value: single);

    procedure Refrescar;
    function GetReadOnly: boolean;
    procedure SetReadOnly(const Value: boolean);
    procedure GuardarIni(pIniFile: string);
  public
    property Titulo:string read GetTitulo write SetTitulo;
    property Valor:single read GetValor write SetValor;
    property ValorMax:single read GetValorMax write SetValorMax;
    property IsReadOnly:boolean read GetReadOnly write SetReadOnly default True;

    procedure Inicializar();
    procedure CalcularPorcentage();

  end;

// Cabeceras de funciones
function GetMaxVolumen(pStreamType:TStreamType):single;
procedure SetVolumen(AValue:single; pStreamType:TStreamType);


implementation

{$R *.fmx}

uses
  IniFiles;


function GetMaxVolumen(pStreamType:TStreamType): single;
{$IFDEF ANDROID}
var
  AudioObj: JObject;
  Audio: JAudioManager;

  //
  function StreamTypeToInteger(pStreamType:TStreamType):integer;
  begin
    case pStreamType of
      stNotificaciones: Result := TJAudioManager.JavaClass.STREAM_NOTIFICATION;
      stMultimedia:     Result := TJAudioManager.JavaClass.STREAM_MUSIC;
      stAlarma:         Result := TJAudioManager.JavaClass.STREAM_ALARM;
      stVoz:            Result := TJAudioManager.JavaClass.STREAM_VOICE_CALL;
    end;
  end;
  //

begin
  // acceso al servicio
  AudioObj:= TAndroidHelper.Activity.getSystemService(TJActivity.JavaClass.AUDIO_SERVICE);
  // acceso a la clase
  Audio := TJAudioManager.Wrap((AudioObj as ILocalObject).GetObjectID);
  Result := Audio.getStreamMaxVolume(StreamTypeToInteger(pStreamType));
{$ELSE}
begin
  Result := 15;
{$ENDIF}
end;

procedure SetVolumen(AValue:single; pStreamType: TStreamType);
{$IFDEF ANDROID}
var
  AudioObj: JObject;
  Audio: JAudioManager;

  //
  function StreamTypeToInteger(pStreamType:TStreamType):integer;
  begin
    case pStreamType of
      stNotificaciones: Result := TJAudioManager.JavaClass.STREAM_NOTIFICATION;
      stMultimedia:     Result := TJAudioManager.JavaClass.STREAM_MUSIC;
      stAlarma:         Result := TJAudioManager.JavaClass.STREAM_ALARM;
      stVoz:            Result := TJAudioManager.JavaClass.STREAM_VOICE_CALL;
    end;
  end;
  //

begin
  // acceso al servicio
  AudioObj:= TAndroidHelper.Activity.getSystemService(TJActivity.JavaClass.AUDIO_SERVICE);
  // acceso a la clase
  Audio := TJAudioManager.Wrap((AudioObj as ILocalObject).GetObjectID);
  Audio.setStreamVolume(StreamTypeToInteger(pStreamType), Trunc(AValue),
      TJAudioManager.JavaClass.FLAG_VIBRATE +
      TJAudioManager.JavaClass.FLAG_PLAY_SOUND +
      TJAudioManager.JavaClass.FLAG_SHOW_UI);
{$ELSE}
begin
  // Nada
  // ...
{$ENDIF}

end;



procedure TfrItem.CalcularPorcentage();
var
  i:integer;
begin
  if (tbValor.Max = 0) then begin
    lblPorcentage.Text := '0 %';
  end
  else begin
    i := Round((tbValor.Value / tbValor.Max) * 100);
    lblPorcentage.Text := IntToStr(i) + ' %';
  end;
end;

function TfrItem.GetReadOnly: boolean;
begin
  Result := not tbValor.Enabled;
end;

function TfrItem.GetTitulo: string;
begin
  Result := lblTitulo.Text;

end;

function TfrItem.GetValor: single;
begin
  Result := tbValor.Value;
end;

function TfrItem.GetValorMax: single;
begin
  Result := tbValor.Max;
end;

procedure TfrItem.GuardarIni(pIniFile: string);
begin

end;

procedure TfrItem.Inicializar();
begin
  tbValor.Value := 0;
  lblPorcentage.Text := '';
  btnAplicar.Visible := False;
  tbValor.Enabled := True;
end;

procedure TfrItem.Refrescar;
begin
  // por si hay que hacer algo...
  CalcularPorcentage();
end;

{
procedure TfrItem.SetFrameType(const Value: TFrameType);
begin
  FFrameType := Value;
  // Si se puede modificar o no hace que est visible el botn de validar
  btnAplicar.Visible := False;
end;
}

procedure TfrItem.SetReadOnly(const Value: boolean);
begin
  tbValor.Enabled := not Value;
end;

procedure TfrItem.SetTitulo(const Value: string);
begin
  lblTitulo.Text := Value;
  Refrescar;
end;

procedure TfrItem.SetValor(const Value: single);
begin
  tbValor.Value := Value;
  Refrescar;
end;

procedure TfrItem.SetValorMax(const Value: single);
begin
  tbValor.Max := Value;
  Refrescar;
end;

procedure TfrItem.tbValorChange(Sender: TObject);
begin
  // recalcularlo
  CalcularPorcentage();
end;

{ TListaPerfiles }
{
procedure TListaPerfiles.AddFrame(pNombre: string; pFrame: TfrItem);
begin
  Self.AddObject(pNombre, pFrame);
end;


function TListaPerfiles.GetFrame(Index: integer): TfrItem;
var
  fr:TfrItem;
begin
  Result := TfrItem(Self.Objects[Index]);
end;

function TListaPerfiles.GetNombre(Index:integer): string;
begin
  Result := Self.Strings[Index];
end;


procedure TListaPerfiles.SetFrame(Index: integer; const Value: TfrItem);
begin
  Self.Frame[Index] := Value;
end;

procedure TListaPerfiles.SetNombre(Index:integer; const Value: string);
begin
  Self.Strings[index] := Value;
end;
}

procedure TPerfiles.AddPerfil(pPerfil: TPerfil);
begin
  Self.AddObject(pPerfil.Nombre, pPerfil);
  SaveToFile;
end;

constructor TPerfiles.Create(pIniFile: String);
begin
  inherited CReate();
  Self.FIniFile := pIniFile;
  Self.OwnsObjects := False;
end;

procedure TPerfiles.DeletePerfil(pPerfil: TPerfil);
var
  index:integer;
  iniF:TIniFile;
  pName:String;
begin
  //  Acceder al perfil
  index := Self.IndexOf(pPerfil.Nombre);
  pName := pPerfil.Nombre;
  if (index > -1) then begin
    // borrarlo de la lista
    Self.Delete(index);
    // Liberar el objeto
    FreeAndNil(pPerfil);
  end;
  // Borrarlo de disco
  if FileExists(Self.FIniFile) then begin
    try
      iniF := TIniFile.Create(FIniFile);
      iniF.EraseSection(pName);
      iniF.UpdateFile;
    finally
      freeAndNil(iniF);
    end;
  end;
  SaveToFile;
end;



function TPerfiles.GetNombreByIndex(Index: integer): string;
begin
  Result := Self.Perfil[Index].Nombre;
end;

function TPerfiles.GetPerfilByIndex(Index: integer): TPerfil;
begin
  Result := TPerfil(Self.Objects[Index]);
end;

function TPerfiles.GetPerfilByName(pName: string): TPerfil;
var
  index:integer;
begin
  Result := nil;
  index := Self.IndexOf(pName);
  if (index > -1) then
    Result := Self.Perfil[index];
end;

procedure TPerfiles.SaveToFile();
var
  iniF:TIniFile;
  I: Integer;
  p:TPerfil;
begin
  if FileExists(Self.FIniFile) then begin
    try
      iniF := TIniFile.Create(FIniFile);

      for I := 0 to (Self.Count - 1) do begin
        p := Self.Perfil[i];
        iniF.WriteFloat(Self.Strings[i], 'Notificaciones', p.VolumeValues.Notificaciones.Value);
        iniF.WriteFloat(Self.Strings[i], 'Multimedia', p.VolumeValues.Multimedia.Value);
        iniF.WriteFloat(Self.Strings[i], 'Alarma', p.VolumeValues.Alarma.Value);
        iniF.WriteFloat(Self.Strings[i], 'Voz', p.VolumeValues.Voz.Value);
      end;
    finally
      iniF.UpdateFile;
      FreeAndNil(iniF);
    end;
  end;
end;



procedure TPerfiles.LoadFromFile();
var
  iniF:TIniFile;
  TS:TStringList;
  vol:TVolumeValues;
  i:integer;
  p:TPerfil;
begin
  if FileExists(Self.FIniFile) then begin
    iniF := TIniFile.Create(FIniFile);
    TS := TStringList.Create();
    try
      iniF.ReadSections(TS);
      // recorrer los perfiles
      for i := 0 to (TS.Count - 1) do begin
        // Crear el perfil
        p := TPerfil.Create();
        // nombre
        p.Nombre := TS[i];
        // valores
        p.FVolumeValues.Notificaciones.Value := iniF.ReadFloat(TS[i], 'Notificaciones', 50);
        p.FVolumeValues.Notificaciones.MaxValue := 100; //GetMaxVolumen(stNotificaciones);
        p.FVolumeValues.Multimedia.Value := iniF.ReadFloat(TS[i], 'Multimedia', 50);
        p.FVolumeValues.Multimedia.MaxValue := 100; //GetMaxVolumen(stMultimedia);
        p.FVolumeValues.Alarma.Value := iniF.ReadFloat(TS[i], 'Alarma', 50);
        p.FVolumeValues.Alarma.MaxValue := 100; //GetMaxVolumen(stAlarma);
        p.FVolumeValues.Voz.Value := iniF.ReadFloat(TS[i], 'Voz', 50);
        p.FVolumeValues.Voz.MaxValue := 100; //GetMaxVolumen(stVoz);

        // Aadirlo a la lista
        Self.AddPerfil(p);
      end;

    finally
      FreeAndNil(iniF);
      FreeAndNil(TS);
    end;
  end;
end;


procedure TPerfiles.UpdatePerfil(pPerfil: TPerfil);
var
  p:TPerfil;
begin
  p := Self.GetPerfilByName(pPerfil.Nombre);
  if Assigned(p) then begin
    p.FVolumeValues.Notificaciones.Value :=  pPerfil.VolumeValues.Notificaciones.Value;
    p.FVolumeValues.Multimedia.Value :=      pPerfil.VolumeValues.Multimedia.Value;
    p.FVolumeValues.Alarma.Value :=          pPerfil.VolumeValues.Alarma.Value;
    p.FVolumeValues.Voz.Value :=             pPerfil.VolumeValues.Voz.Value;
  end;
  SaveToFile;
end;

end.
